"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IA = exports.MSG = exports.EventParams = void 0;
exports.InteractionEvent = InteractionEvent;
exports.AppliedCollectors = AppliedCollectors;
exports.MessageEvent = MessageEvent;
const common_1 = require("@nestjs/common");
const shared_utils_1 = require("@nestjs/common/utils/shared.utils");
const discord_param_type_1 = require("../../factory/discord-param-type");
const event_param_constant_1 = require("./event-param.constant");
/**
 * Extract `Interaction` instance from event params
 */
function InteractionEvent(property, ...pipes) {
    return createPipesEventParamDecorator(discord_param_type_1.DiscordParamType.INTERACTION)(property, ...pipes);
}
exports.EventParams = (0, common_1.createParamDecorator)((data, input) => {
    return input.getArgs();
});
function AppliedCollectors(index, ...pipes) {
    return createPipesEventParamDecorator(discord_param_type_1.DiscordParamType.APPLIED_COLLECTORS)(index.toString(), ...pipes);
}
/**
 * Extract `Message` instance from event params
 */
function MessageEvent(property, ...pipes) {
    return createPipesEventParamDecorator(discord_param_type_1.DiscordParamType.MESSAGE)(property, ...pipes);
}
/**
 * Alias for `MessageEvent` decorator
 */
exports.MSG = MessageEvent;
/**
 * Alias for `InteractionEvent` decorator
 */
exports.IA = InteractionEvent;
const createPipesEventParamDecorator = (paramtype) => (data, ...pipes) => (target, key, index) => {
    const args = Reflect.getMetadata(event_param_constant_1.EVENT_PARAMS_DECORATOR, target.constructor, key) ||
        {};
    const hasParamData = (0, shared_utils_1.isNil)(data) || (0, shared_utils_1.isString)(data);
    const paramData = hasParamData ? data : void 0;
    const paramPipes = hasParamData ? pipes : [data, ...pipes];
    Reflect.defineMetadata(event_param_constant_1.EVENT_PARAMS_DECORATOR, (0, common_1.assignMetadata)(args, paramtype, index, paramData, ...paramPipes), target.constructor, key);
};
//# sourceMappingURL=event-param.decorator.js.map