"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectorExplorer = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const reflect_metadata_provider_1 = require("../../providers/reflect-metadata.provider");
const instantiation_service_1 = require("../../services/instantiation.service");
const interaction_collector_strategy_1 = require("./strategy/interaction-collector.strategy");
const message_collector_strategy_1 = require("./strategy/message-collector.strategy");
const react_collector_strategy_1 = require("./strategy/react-collector.strategy");
let CollectorExplorer = class CollectorExplorer {
    metadataProvider;
    instantiationService;
    metadataScanner;
    reactCollectorStrategy;
    interactionCollectorStrategy;
    messageCollectorStrategy;
    cachedCollectors = new WeakMap();
    constructor(metadataProvider, instantiationService, metadataScanner, reactCollectorStrategy, interactionCollectorStrategy, messageCollectorStrategy) {
        this.metadataProvider = metadataProvider;
        this.instantiationService = instantiationService;
        this.metadataScanner = metadataScanner;
        this.reactCollectorStrategy = reactCollectorStrategy;
        this.interactionCollectorStrategy = interactionCollectorStrategy;
        this.messageCollectorStrategy = messageCollectorStrategy;
    }
    async explore({ instance, methodName, }) {
        const classCollectors = this.metadataProvider.getUseCollectorsDecoratorMetadata(instance) ?? [];
        const methodCollectors = this.metadataProvider.getUseCollectorsDecoratorMetadata(instance, methodName) ?? [];
        if (classCollectors.length === 0 && methodCollectors.length === 0)
            return;
        const hostModule = this.instantiationService.getHostModule(instance);
        const moduleRef = hostModule.getProviderByKey(core_1.ModuleRef).instance;
        const methodCollectorInstances = await this.instantiationService.exploreInstances(methodCollectors, hostModule);
        const methodCollectorInfos = this.getCollectorsInfo(methodCollectorInstances);
        const classType = instance.constructor;
        if (this.cachedCollectors.has(classType))
            this.cachedCollectors.get(classType).methodCollectors[methodName] =
                methodCollectorInfos;
        else {
            const classCollectorInstances = await this.instantiationService.exploreInstances(classCollectors, hostModule);
            const classCollectorInfos = this.getCollectorsInfo(classCollectorInstances);
            this.cachedCollectors.set(classType, {
                methodCollectors: { [methodName]: methodCollectorInfos },
                classCollectors: classCollectorInfos,
                moduleRef,
            });
        }
    }
    async applyCollector(options) {
        const { classType, methodName } = options;
        if (!this.cachedCollectors.has(classType))
            return;
        const { classCollectors, methodCollectors, moduleRef } = this.cachedCollectors.get(classType);
        const collectors = [
            ...classCollectors,
            ...(methodCollectors[methodName] || []),
        ];
        return Promise.all(collectors.map((collector) => collector.strategy.setupCollector(options, collector, moduleRef)));
    }
    getCollectorsInfo(collectorInstances) {
        return collectorInstances.map((collectorInstance) => {
            const { filterMethodName, events } = this.exploreBaseInfo(collectorInstance);
            const reactionMetadata = this.metadataProvider.getReactionCollectorDecoratorMetadata(collectorInstance);
            if (reactionMetadata)
                return {
                    classInstance: collectorInstance,
                    metadata: reactionMetadata,
                    filterMethodName,
                    events,
                    strategy: this.reactCollectorStrategy,
                };
            const messageMetadata = this.metadataProvider.getMessageCollectorDecoratorMetadata(collectorInstance);
            if (messageMetadata)
                return {
                    classInstance: collectorInstance,
                    metadata: messageMetadata,
                    filterMethodName,
                    events,
                    strategy: this.messageCollectorStrategy,
                };
            const interactionMetadata = this.metadataProvider.getInteractionCollectorDecoratorMetadata(collectorInstance);
            if (interactionMetadata)
                return {
                    classInstance: collectorInstance,
                    metadata: interactionMetadata,
                    filterMethodName,
                    events,
                    strategy: this.interactionCollectorStrategy,
                };
        });
    }
    exploreBaseInfo(instance) {
        let filterMethodName;
        const events = {};
        this.getInstanceMethods(instance).forEach((methodName) => {
            let eventMethod = 'on';
            let eventMetadata = this.metadataProvider.getOnCollectEventDecoratorMetadata(instance, methodName);
            if (!eventMetadata) {
                eventMetadata =
                    this.metadataProvider.getOnceCollectEventDecoratorMetadata(instance, methodName);
                eventMethod = 'once';
            }
            if (eventMetadata)
                events[methodName] = {
                    eventName: eventMetadata.event,
                    eventMethod: eventMethod,
                };
            if (filterMethodName)
                return;
            filterMethodName = this.findFilterMethod(instance, methodName);
        });
        return {
            events,
            filterMethodName,
        };
    }
    getInstanceMethods(instance) {
        return this.metadataScanner.scanFromPrototype(instance, Object.getPrototypeOf(instance), (methodName) => methodName);
    }
    findFilterMethod(instance, methodName) {
        const filterMetadata = this.metadataProvider.getFilterDecoratorMetadata(instance, methodName);
        if (filterMetadata)
            return methodName;
    }
};
exports.CollectorExplorer = CollectorExplorer;
exports.CollectorExplorer = CollectorExplorer = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [reflect_metadata_provider_1.ReflectMetadataProvider,
        instantiation_service_1.InstantiationService,
        core_1.MetadataScanner,
        react_collector_strategy_1.ReactCollectorStrategy,
        interaction_collector_strategy_1.InteractionCollectorStrategy,
        message_collector_strategy_1.MessageCollectorStrategy])
], CollectorExplorer);
//# sourceMappingURL=collector.explorer.js.map