"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandExplorer = void 0;
const common_1 = require("@nestjs/common");
const external_context_creator_1 = require("@nestjs/core/helpers/external-context-creator");
const event_param_constant_1 = require("../../decorators/param/event-param.constant");
const discord_param_factory_1 = require("../../factory/discord-param-factory");
const reflect_metadata_provider_1 = require("../../providers/reflect-metadata.provider");
const build_application_command_service_1 = require("../../services/build-application-command.service");
const client_service_1 = require("../../services/client.service");
const option_service_1 = require("../../services/option.service");
let CommandExplorer = class CommandExplorer {
    discordClientService;
    metadataProvider;
    buildApplicationCommandService;
    externalContextCreator;
    optionService;
    constructor(discordClientService, metadataProvider, buildApplicationCommandService, externalContextCreator, optionService) {
        this.discordClientService = discordClientService;
        this.metadataProvider = metadataProvider;
        this.buildApplicationCommandService = buildApplicationCommandService;
        this.externalContextCreator = externalContextCreator;
        this.optionService = optionService;
    }
    discordParamFactory = new discord_param_factory_1.DiscordParamFactory();
    event = 'interactionCreate';
    async explore({ instance }) {
        const metadata = this.metadataProvider.getCommandDecoratorMetadata(instance);
        if (!metadata)
            return;
        const commandData = await this.buildApplicationCommandService.exploreCommand(instance, metadata);
        const mapped = commandData.map(({ name, group, subName, methodName: commandMethodName, instance: commandInstance, }) => {
            if (!commandMethodName || !commandInstance) {
                return;
            }
            const handler = this.externalContextCreator.create(commandInstance, commandInstance[commandMethodName], commandMethodName, event_param_constant_1.EVENT_PARAMS_DECORATOR, this.discordParamFactory);
            return {
                handler,
                commandName: name,
                subCommandName: subName,
                groupName: group,
            };
        });
        this.discordClientService
            .getClient()
            .on(this.event, async (...eventArgs) => {
            await Promise.all(mapped.map(async ({ handler, commandName, subCommandName, groupName }) => {
                const [interaction] = eventArgs;
                if ((!interaction.isChatInputCommand() &&
                    !interaction.isContextMenuCommand()) ||
                    interaction.commandName !== commandName) {
                    return;
                }
                if (interaction.isChatInputCommand() &&
                    ((groupName &&
                        interaction.options.getSubcommandGroup(false) !==
                            groupName) ||
                        (subCommandName &&
                            interaction.options.getSubcommand(false) !==
                                subCommandName)))
                    return;
                try {
                    const returnReply = await handler(...eventArgs, {
                        event: this.event,
                        collectors: [],
                    });
                    if (returnReply) {
                        await interaction.reply(returnReply);
                    }
                }
                catch (exception) {
                    if (exception instanceof common_1.ForbiddenException &&
                        this.optionService.getClientData().isTrowForbiddenException)
                        throw exception;
                }
            }));
        });
    }
};
exports.CommandExplorer = CommandExplorer;
exports.CommandExplorer = CommandExplorer = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [client_service_1.ClientService,
        reflect_metadata_provider_1.ReflectMetadataProvider,
        build_application_command_service_1.BuildApplicationCommandService,
        external_context_creator_1.ExternalContextCreator,
        option_service_1.OptionService])
], CommandExplorer);
//# sourceMappingURL=command.explorer.js.map