"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventExplorer = void 0;
const common_1 = require("@nestjs/common");
const external_context_creator_1 = require("@nestjs/core/helpers/external-context-creator");
const event_param_constant_1 = require("../../decorators/param/event-param.constant");
const discord_param_factory_1 = require("../../factory/discord-param-factory");
const reflect_metadata_provider_1 = require("../../providers/reflect-metadata.provider");
const client_service_1 = require("../../services/client.service");
const option_service_1 = require("../../services/option.service");
let EventExplorer = class EventExplorer {
    metadataProvider;
    discordClientService;
    optionService;
    externalContextCreator;
    logger = new common_1.Logger();
    constructor(metadataProvider, discordClientService, optionService, externalContextCreator) {
        this.metadataProvider = metadataProvider;
        this.discordClientService = discordClientService;
        this.optionService = optionService;
        this.externalContextCreator = externalContextCreator;
    }
    discordParamFactory = new discord_param_factory_1.DiscordParamFactory();
    async explore(options) {
        const { instance, methodName } = options;
        let eventMethod = 'on';
        let metadata = this.metadataProvider.getOnEventDecoratorMetadata(instance, methodName);
        if (!metadata) {
            metadata = this.metadataProvider.getOnceEventDecoratorMetadata(instance, methodName);
            eventMethod = 'once';
            if (!metadata)
                return;
        }
        const { event } = metadata;
        this.logger.log(`Subscribe to event(${eventMethod}): ${event}`, instance.constructor.name);
        const handler = this.externalContextCreator.create(instance, instance[methodName], methodName, event_param_constant_1.EVENT_PARAMS_DECORATOR, this.discordParamFactory);
        this.discordClientService
            .getClient()[eventMethod](event, async (...eventArgs) => {
            try {
                const response = await handler(...eventArgs, {
                    event,
                    collectors: [],
                });
                const [eventArg] = eventArgs;
                if (response && this.hasReply(eventArg)) {
                    await eventArg['reply'](response);
                }
            }
            catch (exception) {
                if (exception instanceof common_1.ForbiddenException &&
                    this.optionService.getClientData().isTrowForbiddenException)
                    throw exception;
            }
        });
    }
    hasReply(value) {
        return typeof value['reply'] === 'function';
    }
};
exports.EventExplorer = EventExplorer;
exports.EventExplorer = EventExplorer = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [reflect_metadata_provider_1.ReflectMetadataProvider,
        client_service_1.ClientService,
        option_service_1.OptionService,
        external_context_creator_1.ExternalContextCreator])
], EventExplorer);
//# sourceMappingURL=event.explorer.js.map