"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OptionExplorer = void 0;
const common_1 = require("@nestjs/common");
const discord_js_1 = require("discord.js");
const param_type_1 = require("../../definitions/types/param.type");
const reflect_metadata_provider_1 = require("../../providers/reflect-metadata.provider");
let OptionExplorer = class OptionExplorer {
    metadataProvider;
    constructor(metadataProvider) {
        this.metadataProvider = metadataProvider;
    }
    explore(dtoType) {
        const optionMetadata = {};
        const dtoInstance = new dtoType();
        Object.keys(dtoInstance).map((propertyKey) => {
            const paramDecoratorMetadata = this.metadataProvider.getParamDecoratorMetadata(dtoType, propertyKey);
            if (!paramDecoratorMetadata) {
                throw new Error('@Param decorator is not set');
            }
            const channelTypes = this.getChannelOptions(dtoType, propertyKey);
            const applicationOptionType = channelTypes
                ? discord_js_1.ApplicationCommandOptionType.Channel
                : this.getApplicationOptionTypeByArg(dtoInstance, propertyKey, paramDecoratorMetadata);
            optionMetadata[propertyKey] = {
                param: {
                    ...paramDecoratorMetadata,
                    name: paramDecoratorMetadata.name ?? propertyKey,
                    type: applicationOptionType,
                },
                choice: this.getChoiceOptions(dtoType, propertyKey),
                channelTypes,
            };
        });
        return optionMetadata;
    }
    getChoiceOptions(dtoType, propertyKey) {
        const choiceData = this.metadataProvider.getChoiceDecoratorMetadata(dtoType, propertyKey);
        if (!choiceData)
            return;
        const isMap = choiceData instanceof Map;
        const entries = isMap
            ? Array.from(choiceData)
            : Object.entries(choiceData).filter(([key]) => !(key in Object.keys(choiceData)));
        return entries.map(([name, value]) => {
            if (typeof value === 'object') {
                return { name, ...value };
            }
            return { name, value };
        });
    }
    getChannelOptions(dtoType, propertyKey) {
        return this.metadataProvider.getChannelDecoratorMetadata(dtoType, propertyKey);
    }
    getApplicationOptionTypeByArg(instance, propertyKey, argDecoratorOptions) {
        switch (argDecoratorOptions.type) {
            case param_type_1.ParamType.STRING:
                return discord_js_1.ApplicationCommandOptionType.String;
            case param_type_1.ParamType.BOOLEAN:
                return discord_js_1.ApplicationCommandOptionType.Boolean;
            case param_type_1.ParamType.INTEGER:
                return discord_js_1.ApplicationCommandOptionType.Integer;
            case param_type_1.ParamType.NUMBER:
                return discord_js_1.ApplicationCommandOptionType.Number;
            case param_type_1.ParamType.ROLE:
                return discord_js_1.ApplicationCommandOptionType.Role;
            case param_type_1.ParamType.MENTIONABLE:
                return discord_js_1.ApplicationCommandOptionType.Mentionable;
            case param_type_1.ParamType.USER:
                return discord_js_1.ApplicationCommandOptionType.User;
            case param_type_1.ParamType.ATTACHMENT:
                return discord_js_1.ApplicationCommandOptionType.Attachment;
            default: {
                const metatype = this.metadataProvider.getPropertyTypeMetadata(instance, propertyKey);
                if (metatype.name === 'String')
                    return discord_js_1.ApplicationCommandOptionType.String;
                if (metatype.name === 'Boolean')
                    return discord_js_1.ApplicationCommandOptionType.Boolean;
                throw new Error(`Could not determine field type "${propertyKey}" in class "${instance.constructor.name}"`);
            }
        }
    }
};
exports.OptionExplorer = OptionExplorer;
exports.OptionExplorer = OptionExplorer = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [reflect_metadata_provider_1.ReflectMetadataProvider])
], OptionExplorer);
//# sourceMappingURL=option.explorer.js.map