"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var BuildApplicationCommandService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuildApplicationCommandService = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const discord_js_1 = require("discord.js");
const is_sub_command_group_1 = require("../decorators/sub-command-group/is-sub-command-group");
const discord_command_provider_1 = require("../providers/discord-command.provider");
const reflect_metadata_provider_1 = require("../providers/reflect-metadata.provider");
const command_handler_finder_service_1 = require("./command-handler-finder.service");
const dto_service_1 = require("./dto.service");
let BuildApplicationCommandService = BuildApplicationCommandService_1 = class BuildApplicationCommandService {
    moduleRef;
    metadataProvider;
    discordCommandProvider;
    dtoService;
    commandHandlerFinderService;
    constructor(moduleRef, metadataProvider, discordCommandProvider, dtoService, commandHandlerFinderService) {
        this.moduleRef = moduleRef;
        this.metadataProvider = metadataProvider;
        this.discordCommandProvider = discordCommandProvider;
        this.dtoService = dtoService;
        this.commandHandlerFinderService = commandHandlerFinderService;
    }
    async exploreCommand(instance, chatInputCommandOptions) {
        const methodName = await this.getMethodName(chatInputCommandOptions, instance);
        const commandData = this.getBaseApplicationCommandData(chatInputCommandOptions);
        const commandListenersDescribe = [];
        if (commandData.type === discord_js_1.ApplicationCommandType.ChatInput) {
            commandData.options = await Promise.all(chatInputCommandOptions.include.map(async (item) => {
                const subCommandOption = await this.exploreSubCommandOptions(commandData.name, item);
                commandListenersDescribe.push(...subCommandOption.commandListenersDescribe);
                return subCommandOption.applicationCommandData;
            }));
        }
        if (commandListenersDescribe.length === 0) {
            commandListenersDescribe.push({
                name: commandData.name,
                instance,
                methodName,
            });
        }
        const dtoType = await this.dtoService.getDtoMetatype(instance, methodName);
        if (dtoType) {
            const commandParams = this.dtoService.exploreDtoOptions(dtoType);
            if (commandData.type === discord_js_1.ApplicationCommandType.ChatInput)
                commandData.options = commandData.options.concat(commandParams);
        }
        if (common_1.Logger.isLevelEnabled('debug')) {
            common_1.Logger.debug('Slash command options', BuildApplicationCommandService_1.name);
            common_1.Logger.debug(commandData, BuildApplicationCommandService_1.name);
        }
        this.discordCommandProvider.addCommand(instance.constructor, {
            commandData,
            additionalOptions: { forGuild: chatInputCommandOptions.forGuild },
        });
        return commandListenersDescribe;
    }
    async exploreSubCommandOptions(commandName, item) {
        if ((0, is_sub_command_group_1.isSubCommandGroup)(item)) {
            return this.getSubCommandGroupOptions(item(), commandName);
        }
        return this.getSubCommandOptions(item, commandName);
    }
    async getSubCommandGroupOptions({ options: { name, description, nameLocalizations, descriptionLocalizations, }, subCommands, }, commandName) {
        const subCommandOptions = [];
        let commandListenersDescribe = [];
        await Promise.all(subCommands.map(async (subCommandType) => {
            const subCommandOption = await this.getSubCommandOptions(subCommandType, commandName, name);
            subCommandOptions.push(subCommandOption.applicationCommandData);
            commandListenersDescribe = commandListenersDescribe.concat(subCommandOption.commandListenersDescribe);
        }));
        return {
            commandListenersDescribe,
            applicationCommandData: {
                name,
                description,
                type: discord_js_1.ApplicationCommandOptionType.SubcommandGroup,
                options: subCommandOptions,
                nameLocalizations,
                descriptionLocalizations,
            },
        };
    }
    async getSubCommandOptions(subCommandType, commandName, subGroupName) {
        const subCommandInstance = this.moduleRef.get(subCommandType, {
            strict: false,
        });
        const metadata = this.metadataProvider.getSubCommandDecoratorMetadata(subCommandInstance);
        if (!metadata)
            throw new Error(`Passed class is not a subcommand`);
        const methodName = await this.commandHandlerFinderService.searchHandler(subCommandInstance);
        const dtoType = await this.dtoService.getDtoMetatype(subCommandInstance, methodName);
        const applicationSubCommandData = {
            name: metadata.name,
            description: metadata.description,
            type: discord_js_1.ApplicationCommandOptionType.Subcommand,
            nameLocalizations: metadata.nameLocalizations,
            descriptionLocalizations: metadata.descriptionLocalizations,
        };
        if (dtoType) {
            applicationSubCommandData.options =
                this.dtoService.exploreDtoOptions(dtoType);
        }
        return {
            commandListenersDescribe: [
                {
                    name: commandName,
                    subName: metadata.name,
                    group: subGroupName,
                    instance: subCommandInstance,
                    methodName,
                },
            ],
            applicationCommandData: applicationSubCommandData,
        };
    }
    getBaseApplicationCommandData(chatInputCommandOptions) {
        const { name, description, dmPermission, defaultMemberPermissions, type = discord_js_1.ApplicationCommandType.ChatInput, nameLocalizations, descriptionLocalizations, } = chatInputCommandOptions;
        const data = {
            type,
            name,
            description,
            nameLocalizations,
            descriptionLocalizations,
        };
        if (defaultMemberPermissions) {
            data.defaultMemberPermissions = defaultMemberPermissions;
        }
        if (dmPermission) {
            data.dmPermission = dmPermission;
        }
        return data;
    }
    async getMethodName(chatInputCommandOptions, instance) {
        if (chatInputCommandOptions.include &&
            chatInputCommandOptions.include.length !== 0) {
            return;
        }
        return this.commandHandlerFinderService.searchHandler(instance);
    }
};
exports.BuildApplicationCommandService = BuildApplicationCommandService;
exports.BuildApplicationCommandService = BuildApplicationCommandService = BuildApplicationCommandService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.ModuleRef,
        reflect_metadata_provider_1.ReflectMetadataProvider,
        discord_command_provider_1.DiscordCommandProvider,
        dto_service_1.DtoService,
        command_handler_finder_service_1.CommandHandlerFinderService])
], BuildApplicationCommandService);
//# sourceMappingURL=build-application-command.service.js.map