"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@nestjs/core");
const testing_1 = require("@nestjs/testing");
const discord_js_1 = require("discord.js");
const command_decorator_1 = require("../decorators/command/command.decorator");
const handler_decorator_1 = require("../decorators/command/handler/handler.decorator");
const param_decorator_1 = require("../decorators/option/param/param.decorator");
const use_group_1 = require("../decorators/sub-command-group/use-group");
const sub_command_decorator_1 = require("../decorators/sub-command/sub-command.decorator");
const option_explorer_1 = require("../explorers/option/option.explorer");
const discord_command_provider_1 = require("../providers/discord-command.provider");
const reflect_metadata_provider_1 = require("../providers/reflect-metadata.provider");
const build_application_command_service_1 = require("./build-application-command.service");
const command_handler_finder_service_1 = require("./command-handler-finder.service");
const dto_service_1 = require("./dto.service");
describe('Build application command service', () => {
    let testingModule;
    beforeEach(async () => {
        testingModule = await testing_1.Test.createTestingModule({
            providers: [
                build_application_command_service_1.BuildApplicationCommandService,
                reflect_metadata_provider_1.ReflectMetadataProvider,
                discord_command_provider_1.DiscordCommandProvider,
                dto_service_1.DtoService,
                option_explorer_1.OptionExplorer,
                command_handler_finder_service_1.CommandHandlerFinderService,
                core_1.MetadataScanner,
            ],
        }).compile();
    });
    it('should explore slash command and add it to DiscordCommandProvider', async () => {
        const buildApplicationCommand = testingModule.get(build_application_command_service_1.BuildApplicationCommandService);
        const commandOptions = {
            name: 'play',
            type: discord_js_1.ApplicationCommandType.ChatInput,
            description: 'Play command',
        };
        let PlayCommand = class PlayCommand {
            onPlayCommand() {
                return 'ok';
            }
        };
        __decorate([
            (0, handler_decorator_1.Handler)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], PlayCommand.prototype, "onPlayCommand", null);
        PlayCommand = __decorate([
            (0, command_decorator_1.Command)(commandOptions)
        ], PlayCommand);
        const commandInstance = new PlayCommand();
        jest
            .spyOn(testingModule.get(command_handler_finder_service_1.CommandHandlerFinderService), 'searchHandler')
            .mockResolvedValueOnce(commandInstance.onPlayCommand.name);
        const discordCommandProvider = testingModule.get(discord_command_provider_1.DiscordCommandProvider);
        jest.spyOn(discordCommandProvider, 'addCommand');
        const actual = await buildApplicationCommand.exploreCommand(commandInstance, commandOptions);
        expect(actual).toStrictEqual(expect.arrayContaining([
            {
                name: 'play',
                instance: commandInstance,
                methodName: 'onPlayCommand',
            },
        ]));
        expect(discordCommandProvider.addCommand).toBeCalledWith(PlayCommand, {
            additionalOptions: { forGuild: undefined },
            commandData: {
                defaultMemberPermissions: undefined,
                description: 'Play command',
                descriptionLocalizations: undefined,
                dmPermission: undefined,
                name: 'play',
                nameLocalizations: undefined,
                options: [],
                type: 1,
            },
        });
    });
    it('should explore slash command with options and add it to DiscordCommandProvider', async () => {
        const buildApplicationCommand = testingModule.get(build_application_command_service_1.BuildApplicationCommandService);
        const commandOptions = {
            name: 'play',
            type: discord_js_1.ApplicationCommandType.ChatInput,
            description: 'Play command',
        };
        class PlayOptions {
            track;
        }
        __decorate([
            (0, param_decorator_1.Param)({ description: 'Track name' }),
            __metadata("design:type", String)
        ], PlayOptions.prototype, "track", void 0);
        let PlayCommand = class PlayCommand {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            onPlayCommand(dto) {
                return 'ok';
            }
        };
        __decorate([
            (0, handler_decorator_1.Handler)()
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            ,
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [PlayOptions]),
            __metadata("design:returntype", void 0)
        ], PlayCommand.prototype, "onPlayCommand", null);
        PlayCommand = __decorate([
            (0, command_decorator_1.Command)(commandOptions)
        ], PlayCommand);
        const commandInstance = new PlayCommand();
        jest
            .spyOn(testingModule.get(command_handler_finder_service_1.CommandHandlerFinderService), 'searchHandler')
            .mockResolvedValueOnce(commandInstance.onPlayCommand.name);
        const discordCommandProvider = testingModule.get(discord_command_provider_1.DiscordCommandProvider);
        jest.spyOn(discordCommandProvider, 'addCommand');
        const actual = await buildApplicationCommand.exploreCommand(commandInstance, commandOptions);
        expect(actual).toStrictEqual(expect.arrayContaining([
            {
                name: 'play',
                instance: commandInstance,
                methodName: 'onPlayCommand',
            },
        ]));
        expect(discordCommandProvider.addCommand).toBeCalledWith(PlayCommand, {
            additionalOptions: { forGuild: undefined },
            commandData: {
                defaultMemberPermissions: undefined,
                description: 'Play command',
                descriptionLocalizations: undefined,
                dmPermission: undefined,
                name: 'play',
                nameLocalizations: undefined,
                options: [
                    {
                        autocomplete: undefined,
                        channelTypes: undefined,
                        choices: undefined,
                        description: 'Track name',
                        descriptionLocalizations: undefined,
                        maxLength: undefined,
                        maxValue: undefined,
                        minLength: undefined,
                        minValue: undefined,
                        name: 'track',
                        nameLocalizations: undefined,
                        required: false,
                        type: 3,
                    },
                ],
                type: 1,
            },
        });
    });
    describe('Subcommand', () => {
        class EmailDto {
            email;
        }
        __decorate([
            (0, param_decorator_1.Param)({ description: 'Your email' }),
            __metadata("design:type", String)
        ], EmailDto.prototype, "email", void 0);
        let EmailSubCommand = class EmailSubCommand {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            onEmailCommand(dto) {
                return 'ok';
            }
        };
        __decorate([
            (0, handler_decorator_1.Handler)()
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            ,
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [EmailDto]),
            __metadata("design:returntype", void 0)
        ], EmailSubCommand.prototype, "onEmailCommand", null);
        EmailSubCommand = __decorate([
            (0, sub_command_decorator_1.SubCommand)({
                name: 'email',
                description: 'Register by subcommand',
            })
        ], EmailSubCommand);
        let BaseInfoSubCommand = class BaseInfoSubCommand {
            onBaseInfoCommand() {
                return 'ok';
            }
        };
        __decorate([
            (0, handler_decorator_1.Handler)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], BaseInfoSubCommand.prototype, "onBaseInfoCommand", null);
        BaseInfoSubCommand = __decorate([
            (0, sub_command_decorator_1.SubCommand)({
                name: 'base-info',
                description: 'Return base info about user',
            })
        ], BaseInfoSubCommand);
        beforeEach(async () => {
            testingModule = await testing_1.Test.createTestingModule({
                providers: [
                    build_application_command_service_1.BuildApplicationCommandService,
                    reflect_metadata_provider_1.ReflectMetadataProvider,
                    discord_command_provider_1.DiscordCommandProvider,
                    dto_service_1.DtoService,
                    option_explorer_1.OptionExplorer,
                    command_handler_finder_service_1.CommandHandlerFinderService,
                    core_1.MetadataScanner,
                    EmailSubCommand,
                    BaseInfoSubCommand,
                ],
            }).compile();
        });
        it('should explore slash sub command and add it to DiscordCommandProvider', async () => {
            const buildApplicationCommand = testingModule.get(build_application_command_service_1.BuildApplicationCommandService);
            const commandOptions = {
                name: 'reg',
                description: 'User registration',
                include: [
                    (0, use_group_1.UseGroup)({ name: 'type', description: 'Registration type' }, EmailSubCommand),
                    BaseInfoSubCommand,
                ],
            };
            let RegistrationCommand = class RegistrationCommand {
            };
            RegistrationCommand = __decorate([
                (0, command_decorator_1.Command)(commandOptions)
            ], RegistrationCommand);
            const commandInstance = new RegistrationCommand();
            const discordCommandProvider = testingModule.get(discord_command_provider_1.DiscordCommandProvider);
            jest.spyOn(discordCommandProvider, 'addCommand');
            const actual = await buildApplicationCommand.exploreCommand(commandInstance, commandOptions);
            expect(actual).toStrictEqual(expect.arrayContaining([
                {
                    group: 'type',
                    instance: testingModule.get(EmailSubCommand),
                    methodName: 'onEmailCommand',
                    name: 'reg',
                    subName: 'email',
                },
                {
                    group: undefined,
                    instance: testingModule.get(BaseInfoSubCommand),
                    methodName: 'onBaseInfoCommand',
                    name: 'reg',
                    subName: 'base-info',
                },
            ]));
            expect(discordCommandProvider.addCommand).toBeCalledWith(RegistrationCommand, {
                additionalOptions: { forGuild: undefined },
                commandData: {
                    defaultMemberPermissions: undefined,
                    description: 'User registration',
                    descriptionLocalizations: undefined,
                    dmPermission: undefined,
                    name: 'reg',
                    nameLocalizations: undefined,
                    options: [
                        {
                            description: 'Registration type',
                            descriptionLocalizations: undefined,
                            name: 'type',
                            nameLocalizations: undefined,
                            options: [
                                {
                                    description: 'Register by subcommand',
                                    descriptionLocalizations: undefined,
                                    name: 'email',
                                    nameLocalizations: undefined,
                                    options: [
                                        {
                                            autocomplete: undefined,
                                            channelTypes: undefined,
                                            choices: undefined,
                                            description: 'Your email',
                                            descriptionLocalizations: undefined,
                                            maxLength: undefined,
                                            maxValue: undefined,
                                            minLength: undefined,
                                            minValue: undefined,
                                            name: 'email',
                                            nameLocalizations: undefined,
                                            required: false,
                                            type: 3,
                                        },
                                    ],
                                    type: 1,
                                },
                            ],
                            type: 2,
                        },
                        {
                            description: 'Return base info about user',
                            descriptionLocalizations: undefined,
                            name: 'base-info',
                            nameLocalizations: undefined,
                            type: 1,
                        },
                    ],
                    type: 1,
                },
            });
        });
    });
});
//# sourceMappingURL=build-application-command.spec.js.map