"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var RegisterCommandService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.RegisterCommandService = void 0;
const common_1 = require("@nestjs/common");
const discord_js_1 = require("discord.js");
const rxjs_1 = require("rxjs");
const inject_discord_client_decorator_1 = require("../decorators/client/inject-discord-client.decorator");
const discord_command_provider_1 = require("../providers/discord-command.provider");
const option_service_1 = require("./option.service");
let RegisterCommandService = RegisterCommandService_1 = class RegisterCommandService {
    discordCommandProvider;
    client;
    discordOptionService;
    logger = new common_1.Logger(RegisterCommandService_1.name);
    constructor(discordCommandProvider, client, discordOptionService) {
        this.discordCommandProvider = discordCommandProvider;
        this.client = client;
        this.discordOptionService = discordOptionService;
    }
    async register() {
        const options = this.discordOptionService.getClientData();
        this.client.on('ready', () => this.registerCommands(this.client, options));
    }
    async registerCommands(client, { registerCommandOptions }) {
        const commands = this.discordCommandProvider.getAllCommands();
        if (commands.size === 0)
            return;
        const commandList = Array.from(commands.values());
        if (!client.application?.owner)
            await client.application?.fetch();
        await Promise.all(registerCommandOptions.map(async (commandOptions) => {
            const { allowFactory, trigger } = commandOptions;
            const register = async () => {
                if (allowFactory) {
                    client.on('messageCreate', async (message) => {
                        if (!allowFactory(message, commandList))
                            return;
                        await (0, rxjs_1.lastValueFrom)(this.groupCommandsByGuildId(commandList, commandOptions).pipe((0, rxjs_1.tap)((commandInfo) => this.setupCommands(commandInfo))));
                    });
                }
                else {
                    await (0, rxjs_1.lastValueFrom)(this.groupCommandsByGuildId(commandList, commandOptions).pipe((0, rxjs_1.tap)((commandInfo) => this.setupCommands(commandInfo))));
                }
            };
            if (trigger)
                trigger(commandList).subscribe({
                    next: async () => {
                        await register();
                    },
                });
            else
                await register();
        }));
    }
    async dropMissingCommands(existCommands, commandList) {
        const deleteCommands = existCommands.filter((existCommand) => !commandList.find((newCommand) => existCommand.name === newCommand.name &&
            existCommand.type === newCommand.type));
        await Promise.all(deleteCommands.map((command) => command.delete()));
    }
    splitByCommands(existCommandList, commandList) {
        return commandList.reduce((result, newCommand) => {
            const alreadyExist = existCommandList.find((existCommand) => newCommand.name === existCommand.name &&
                newCommand.type === existCommand.type);
            alreadyExist
                ? result[1].push({
                    existCommand: alreadyExist,
                    newCommand,
                })
                : result[0].push(newCommand);
            return result;
        }, [[], []]);
    }
    groupCommandsByGuildId(appCommandData, registerCommandOptions) {
        return (0, rxjs_1.from)(appCommandData).pipe((0, rxjs_1.map)((commandData) => this.defineGuildIdForCommand(commandData, registerCommandOptions)), (0, rxjs_1.groupBy)(({ registerCommandOptions }) => registerCommandOptions.forGuild), (0, rxjs_1.mergeMap)((group) => group.pipe(this.unionGroupByCommandData())));
    }
    defineGuildIdForCommand({ commandData, additionalOptions }, registerCommandOptions) {
        if (additionalOptions.forGuild) {
            return {
                commandData,
                registerCommandOptions: {
                    ...registerCommandOptions,
                    forGuild: additionalOptions.forGuild,
                },
            };
        }
        return {
            commandData,
            registerCommandOptions,
        };
    }
    unionGroupByCommandData() {
        return (0, rxjs_1.reduce)((acc, { commandData, registerCommandOptions }) => {
            acc.commandList = acc.commandList.concat(commandData);
            acc.registerCommandOptions = registerCommandOptions;
            return acc;
        }, {
            commandList: [],
        });
    }
    async setupCommands({ commandList, registerCommandOptions: { forGuild, removeCommandsBefore }, }) {
        // Fetch guild or global commands
        const existCommandList = await this.client.application.commands.fetch(forGuild && {
            guildId: forGuild,
        });
        // Remove if needed
        if (removeCommandsBefore) {
            await this.dropMissingCommands(existCommandList, commandList);
            this.logger.log(`All ${forGuild ? 'guild' : 'global'} commands removed!`);
        }
        let registeredCommands = [];
        const [createCommands, editCommands] = this.splitByCommands(existCommandList, commandList);
        // Edit existing commands
        if (editCommands.length !== 0) {
            const editedCommands = await Promise.all(editCommands.map(({ existCommand, newCommand }) => existCommand.edit(newCommand)));
            registeredCommands = registeredCommands.concat(editedCommands);
        }
        // Create new commands
        if (createCommands.length !== 0) {
            const createdCommands = await this.client.application.commands.set(commandList, forGuild);
            registeredCommands = registeredCommands.concat(Array.from(createdCommands.values()));
        }
        this.logger.log(`All ${forGuild ? 'guild' : 'global'} commands are registered!`);
        return registeredCommands;
    }
};
exports.RegisterCommandService = RegisterCommandService;
exports.RegisterCommandService = RegisterCommandService = RegisterCommandService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, inject_discord_client_decorator_1.InjectDiscordClient)()),
    __metadata("design:paramtypes", [discord_command_provider_1.DiscordCommandProvider,
        discord_js_1.Client,
        option_service_1.OptionService])
], RegisterCommandService);
//# sourceMappingURL=register-command.service.js.map